/**
 * Define api of apeman-api-db
 * @function api
 * @param {Apemandb} db - Instance of Apemandb class.
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @returns {Object} - API configuration
 */
'use strict'

const co = require('co')
const aslogger = require('aslogger')

const spec = require('./spec')

/** @lends api */
function api (db, options = {}) {
  let { logging = false } = options
  let logger = aslogger({ disabled: !logging })

  return {
    /**
     * Test the reachability of the api.
     * @method ping
     * @param {string} [pong='pong'] - Pong message to return
     * @returns {string} - Pong message
     */
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },

    /**
     * Setup data base
     * @method setup
     * @param {string} rootUsername - Username of root user
     * @param {string} rootPassword - Passowrd of root user
     * @returns {Promise}
     */
    setup (rootUsername, rootPassword) {
      const s = this
      return co(function * () {
        yield db.setup(rootUsername, rootPassword)
      })
    },

    /**
     * Sync database tables
     * @method sync
     * @param {Object} [options={}] - Optional settings
     * @returns {Promise}
     */
    sync (options = {}) {
      const s = this
      return co(function * () {
        yield db.sync(options)
      })
    },

    /**
     * Reset database
     * @method reset
     * @returns {Promise}
     */
    reset () {
      const s = this
      return co(function * () {
        yield db.reset()
      })
    },

    /**
     * Describe database
     * @method describe
     * @returns {Promise.<Object>}
     */
    describe () {
      const s = this
      return co(function * () {
        return yield db.describe()
      })
    },

    /**
     * Close database
     * @method close
     * @returns {Promise}
     */
    close () {
      const s = this
      return co(function * () {
        return Promise.resolve(db.close())
      })
    },

    /**
     * Generate database data
     * @method seed
     * @param {Object} [options={}] - Optional settings
     * @returns {Promise}
     */
    seed (options = {}) {
      const s = this
      return co(function * () {
        yield db.seed(options)
      })
    },

    /**
     * Drop database
     * @method drop
     * @param {Object} [options={}] - Optional settings
     * @returns {Promise}
     */
    drop (options = {}) {
      const s = this
      return co(function * () {
        yield db.drop(options)
      })
    },

    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { state } = s
      return co(function * () {
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },
    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },
    /**
     * Method specifications
     */
    $spec: spec
  }
}

module.exports = api
